/***************************************************************************
 *            Firmware_Adapterbox.c
 *
 *  Mo May 31 16:30:00 2010
 *  Copyright  2010  Albrecht Lohoefener
 *  albrechtloh@gmx.de
 ****************************************************************************/

/*! \mainpage Firmware Adapterbox
 *
 * \section intro_sec Einleitung
 *
 * Diese Software laeuft auf einem AVR ATMega123 (16 MHz) Developmentboard oder auf einem ATTiny85 (1 MHz) und hat die Aufgabe Impulse die von einem
 * Herzfrequenzmessgurt empfangen werdem per FSK Modulation an einem PC oder Mobiltelefon zu senden.
 * Die Herzfrequenz ist in bpm (Beats per Minute) angegeben.
 *
 * \section install_sec Installation
 *
 * \subsection step1 Step 1: Opening the box
 *
 * \section aufbau_sec Aufbau der Firmware
 * Die gesamte Firmware ist Ereigniss gesteuert, d. h. in der Endlossschleife stehen keine Anweisungen. Zeitlich gesteuerte Ereignisse werden von einem Softwaretimer erzeugt. ALs Zeitbasis fuer diesen Softwaretimer wird ein Hardwaretimer verwendet. Andere Ereignisse koennen das Empfangen von Impulsen des Polarempfaengers (externe Interrupt) sein.
 */


/**
* \file
* Diese Datei enthaelt die Initialisierung der Hardware und das Hauptprogramm. 
*/


#ifdef __AVR_ATmega128__
#ifndef F_CPU
/* Definiere F_CPU, wenn F_CPU nicht bereits vorher definiert 
   (z.B. durch UEbergabe als Parameter zum Compiler innerhalb 
   des Makefiles). Zusaetzlich Ausgabe einer Warnung, die auf die
   "nachtraegliche" Definition hinweist */
#warning "F_CPU war noch nicht definiert, wird nun mit 16000000UL definiert"
#define F_CPU 16000000UL     /* Quarz mit 16.000 Mhz */
#endif
#endif

#ifdef __AVR_ATtiny85__
#ifndef F_CPU
/* Definiere F_CPU, wenn F_CPU nicht bereits vorher definiert
   (z.B. durch UEbergabe als Parameter zum Compiler innerhalb
   des Makefiles). Zusaetzlich Ausgabe einer Warnung, die auf die
   "nachtraegliche" Definition hinweist */
#warning "F_CPU war noch nicht definiert, wird nun mit 1000000UL definiert"
#define F_CPU 8000000UL     /* Quarz mit 16.000 Mhz */
#endif
#endif

#include <avr/io.h>

#include "PWM.h"
#include "LED.h"
#include "Timer.h"
#include "Interrupt.h"
#include "FSK.h"
#include "HeartRate.h"

#ifdef TESTFUNCTIONS
#include "TestFunct.h"
#include "TestKey.h"
#include "TestPin.h"
#endif

// Lokale Prototypen
 /**
  * \brief  Initialisierungsroutine fuer die Hardware (PWM, Timer, LED, HeartRate Empfaenger und Interrupts)
  *
  * \param 	keine
  * \return	0
  *
  */
void Initializing(void);

 /**
  * \brief  Hauptroutine, Initalisierung, Starten der Senderoutine, Setzen der Software Timer. Am Ende der Routine steht eine Endlossschleife in der keine Funktion enthalten ist.
  *
  * \param 	keine
  * \return	0
  *
  */
int main(void)
{
	// Initialisieren
	Initializing();

	// Starten der FSK Modulation
	FSKSendStart();	

	// Endlossschleife
	while(1);

	return 0;
}

void Initializing(void)
{
	// PWM initialisieren
	PWMInit();

	// LED initialisieren
	LEDInit();

	// Timer initialisieren
	TimerInit();

	// Herzfrequenzzaehler initialisieren
	HeartRateInit();

#ifdef TESTFUNCTIONS
	// Testtaster initialisieren
	TestKeyInit();
	//TestPinInit();
#endif

	// Alle Interrupts anschalten
	InterruptsOn();
}
