/***************************************************************************
 *            HeartRate.h
 *
 *  Sun Jun 06 13:40:00 2010
 *  Copyright  2010  Albrecht Lohoefener
 *  albrechtloh@gmx.de
 ****************************************************************************/

#ifndef HEARTRATE_H

#include <avr/io.h>

/**
*	Groesse des Puffers des Herzfrequenzpuffers
*/
#define HEARTRATEBUFFERSIZE 8

/**
*	Puffer fuer Zeitdauer eines Pulses
*/
typedef struct HeartRateValueBuffer_t
{
	uint16_t Time[HEARTRATEBUFFERSIZE];
	uint8_t WritePointer;
}HeartRateValueBuffer_t;


/**
*	Zeitzaehler um die Dauer zwischen zwei Impulse zu messen
*/
uint16_t HeartRateTimeCounter;

 /**
  * \brief  Initialisiert das Modul HeartRate
  *
  * \param 	keine
  * \return	keine
  *
  */
void HeartRateInit(void);

 /**
  * \brief  Berechnet die Herzfrequenz in Impulse pro Minute aus den Zeiten, die im HeartRateBuffer abgespeichert sind
  *        	Alle Werte im Puffer werden gemittelt und in Impulse pro Minute umgerechnet
  *			Sollte eine 0 im Puffer vorhanden sein, so wird der Puffer nicht verwendet
  *			Ist eine bestimmte Zeit kein Impuls registriert wurden, so wird eine 0 in den Buffer geschrieben und die LED blinkt schnell
  *
  * \param 	keine
  * \return	keine
  *
  */
void HeartRateCalculate(void);

 /**
  * \brief  Schreibt die gemessene Zeit zwischen zwei Impulse einen Ringpuffer
  *         UEberprueft die Plausabilitaet des Ergebnisses.
  *			Laesst die LED kurz aufleuchten
  *
  * \param 	keine
  * \return	keine
  *
  */
void HeartRateInterrupt(void);

 /**
  * \brief  Erhoeht einen Zaehler um die Zeit zwischen zwei Pulse zu messen
  *
  * \param 	keine
  * \return	keine
  *
  */
void HeartRateIncTime(void);

#define HEARTRATE_H
#endif
