/***************************************************************************
 *            Interrupt.c
 *
 *  Fr Jun 04 12:50:00 2010
 *  Copyright  2010  Albrecht Lohoefener
 *  albrechtloh@gmx.de
 ****************************************************************************/

/**
* \file
* Regelt den Zugriff, initialisiert und handelt einkommende Interrupts.
*/

#include <avr/interrupt.h> 

#include "Interrupt.h"
#include "Timer.h"
#include "HeartRate.h"

#ifdef TESTFUNCTIONS
#include "TestKey.h"
#endif

int InterruptsOn(void)
{
	sei();

	return 0;
}

int InterruptsOff(void)
{
	cli();

	return 0;
}

void InterruptHeartRateInit(void)
{
	#ifdef __AVR_ATmega128__
	// Interrupt 0 auf steigende Flanke konfigurieren
	EICRA |= (1 << ISC00) | (1 << ISC01);

	// Interrupt 0 einschalten
	EIMSK |= (1 << INT0);
	#endif

	#ifdef __AVR_ATtiny85__
	// Interrupt 0 auf steigende Flanke konfigurieren
	MCUCR |= (1 << ISC00) | (1 << ISC01);

	// Interrupt 0 einschalten
	GIMSK |= (1 << INT0);
	#endif
}

#ifdef TESTFUNCTIONS
void InterruptTestKeyInit(void)
{
	#ifdef __AVR_ATmega128__
	// Interrupt 1 auf steigende Flanke konfigurieren
	EICRA |= (1 << ISC10) | (1 << ISC11);

	// Interrupt 1 einschalten
	EIMSK |= (1 << INT1);
	#endif
}
#endif

// Interrupt ISR fuer SFTimer Basis
#ifdef __AVR_ATmega128__
ISR(TIMER2_COMP_vect)
{
	TimerInterrupt();
}
#endif
#ifdef __AVR_ATtiny85__
ISR(TIMER0_COMPA_vect)
{
	TimerInterrupt();
}
#endif


ISR(INT0_vect)
{
	HeartRateInterrupt();
}

#ifdef TESTFUNCTIONS
ISR(INT1_vect)
{
	TestKeyInterrupt();
}
#endif

