/***************************************************************************
 *            RingBuffer.c
 *
 *  Fr Jun 04 21:10:00 2010
 *  Copyright  2010  Albrecht Lohoefener
 *  albrechtloh@gmx.de
 ****************************************************************************/

/**
* \file
* Stellt einen Ringbuffer ohne UEberlaufdetektion bereit. Sollte der Ringpuffer voll werden, so werden die alten Werte ueberschrieben.
*/ 

#include <avr/io.h>

#include "RingBuffer.h"

/**
*	Groesse des RingBuffers in Byte
*/
#define RingBufferSize 16

// Globale Variablen fuer Ring Buffer
volatile uint8_t RingBufferData[RingBufferSize];
volatile uint8_t RingBufferReadPointer = 0;
volatile uint8_t RingBufferWritePointer = 0;

int RingBufferSetValue(char Value)
{
	// Wert in den Ring Buffer schreiben
	RingBufferData[RingBufferWritePointer] = Value;

	// WritePointer um eins erhoehen
	RingBufferWritePointer++;

	// Testen ob Ring Buffer zu Ende ist
	if(RingBufferWritePointer >= RingBufferSize)
		RingBufferWritePointer = 0;

	return 0;
}

char RingBufferGetValue(void)
{
	static uint8_t RingBufferReadValue = 0;

	// Testen ob Ring Buffer leer ist
	if(RingBufferReadPointer == RingBufferWritePointer)
	{
		// Ring Buffer ist leer
		RingBufferReadValue = 0;
	}	
	else
	{
		// Wert aus Ring Buffer lesen
		RingBufferReadValue = RingBufferData[RingBufferReadPointer];

		// ReadPointer um eins erhoehen
		RingBufferReadPointer++;
	}

	// Testen ob Ring Buffer zu Ende ist
	if(RingBufferReadPointer >= RingBufferSize)
		RingBufferReadPointer = 0;

	return RingBufferReadValue;
}
