/***************************************************************************
 *            Timer.c
 *
 *  Fr Jun 04 12:42:00 2010
 *  Copyright  2010  Albrecht Lohoefener
 *  albrechtloh@gmx.de
 ****************************************************************************/

/**
* \file
* Diese Datei stellt einen Softwaretimer bereit. Als Basis wird ein Hardwaretimer verwendet. Jeder Softwaretimer besteht aus einer Struktur in der Timeoutwert, Momentanwert und die Callbackfunktion gespeichert ist. Ist der Timeoutwert herrunter gezählt, so wird die Callbackfunktion gerufen und der Timeoutwert von vorne herruntergezählt.
*/ 

#include <avr/io.h>

#include "Timer.h"

// SFTimerstruktur aufbauen
Timer_t SFTimer[MAXSFTIMER];

int TimerInit(void)
{
	uint16_t counter_value = 0;

	#ifdef __AVR_ATmega128__
	// WGM21 CTC Mode
	TCCR2 = (1 << WGM21);

	// Prescaler setzen
	TCCR2 |= PRESCALERTIMER2_C_ATMEGA128;

	// Overflow Interrupt erlauben
	TIMSK |= (1<<OCIE2);

	// Berechnung des Zaehlwertes
	// TODO: Diese Rechnung mit Precompiler erledigen
	counter_value = (F_CPU / (uint32_t)((uint32_t)PRESCALERTIMER2_ATMEGA128 * (uint32_t)TIMERFREQUENCY)) - 1;

	// Zaehlerwert setzen
	OCR2 = counter_value;
	#endif

	#ifdef __AVR_ATtiny85__
	// WGM01 CTC Mode
	TCCR0A = (1 << WGM01);

	// Prescaler setzen
	TCCR0B |= PRESCALERTIMER0_C_ATTINY85;

	// Overflow Interrupt erlauben
	TIMSK |= (1<<OCIE0A);

	// Berechnung des Zaehlwertes	
	// TODO: Diese Rechnung mit Precompiler erledigen
	counter_value = (F_CPU / (uint32_t)((uint32_t)PRESCALERTIMER0_ATTINY85 * (uint32_t)TIMERFREQUENCY)) - 1;

	// Zaehlerwert setzen
	OCR0A = counter_value;
	#endif
	return 0;
}


int TimerSet(TimerName_t TimerHandle, TimerMode_t TimerMode, uint16_t Timeoutms, Callback_t Callback)
{
	// Testen ob TimerHandle nicht groesser als maximale Softwaretimer ist
	if(Timeoutms > 0 && TimerHandle <= MAXSFTIMER)
	{
		SFTimer[TimerHandle].WaitTime = Timeoutms  * (TIMERFREQUENCY / 1000);
		SFTimer[TimerHandle].CallbackFunction = Callback;
		SFTimer[TimerHandle].TimerMode = TimerMode;
		SFTimer[TimerHandle].ActTime = 1;
		SFTimer[TimerHandle].TimerStatus = stop;
	}
	else
		return -1;

	return 0;
}

void TimerStart(TimerName_t TimerHandle)
{
	// Timerhandle nicht groesser als maximale Timeranzahl
	if(TimerHandle <= MAXSFTIMER)
	{
		SFTimer[TimerHandle].TimerStatus = running;
	}
}

void TimerStop(TimerName_t TimerHandle)
{
	// Timerhandle nicht groesser als maximale Timeranzahl
	if(TimerHandle <= MAXSFTIMER)
	{
		SFTimer[TimerHandle].TimerStatus = stop;

		// Softwaretimer ruecksetzen
		SFTimer[TimerHandle].ActTime = 1;
	}
}

void TimerInterrupt(void)
{	
	int i;

	for(i=0;i<MAXSFTIMER;i++)
	{
		if(SFTimer[i].WaitTime!=0 && SFTimer[i].TimerStatus == running)
		{
			if(SFTimer[i].ActTime >= SFTimer[i].WaitTime)
			{
				// Softwaretimer anhalten wenn single
				if(SFTimer[i].TimerMode == single) SFTimer[i].TimerStatus = stop;
				
				// Softwaretimer neu starten
				SFTimer[i].ActTime = 1;
	
				// Callback Funktion rufen
				SFTimer[i].CallbackFunction();
			}
			else
			{
				SFTimer[i].ActTime++ ;
			}
		}
	}
}


