/***************************************************************************
 *            Timer.h
 *
 *  Fr Jun 04 12:41:00 2010
 *  Copyright  2010  Albrecht Lohoefener
 *  albrechtloh@gmx.de
 ****************************************************************************/

#ifndef Timer_H

/**
*	Vorteiler fuer den BasisTimer
*/
#define PRESCALERTIMER2_ATMEGA128 64
#define PRESCALERTIMER0_ATTINY85 64
/**
*	Register Werte fuer Prescaler
*/
#define PRESCALERTIMER2_C_ATMEGA128 ((1 << CS21) | (1 << CS20)) // Prescaler = 64
#define PRESCALERTIMER0_C_ATTINY85 ((1 << CS01)| (1 << CS00))  // Prescaler = 64

/**
*	Timeoutfrequenz des Timers
*/
#define TIMERFREQUENCY 1000

/**
*	Datentyp fuer Callback Funktion
*/
typedef void (*Callback_t)();

/**
*	Datentyp Modus des Timers
*/
typedef enum {single,period}TimerMode_t;

/**
*	Datentyp Status des Timers
*/
typedef enum {running,stop}TimerStatus_t;

/**
*	Datentyp fuer Timer Name
*/
typedef enum {LED=0,FSK,HeartRateCalc,HeartRateTime,HeartRateLED,TestKeyDebounce}TimerName_t;

/**
*	Anzahl der Software Timer
*/
#define MAXSFTIMER 6

/**
*	Timer Struktur
*/
typedef struct Timer_t
{
	uint16_t WaitTime;
	uint16_t ActTime;
	Callback_t CallbackFunction;
	TimerMode_t TimerMode;
	TimerStatus_t TimerStatus;
}Timer_t;

 /**
  * \brief  Initialisierungsroutine fuer den Softwaretimer
  *
  * \return	0
  *
  */
int TimerInit(void);

 /**
  * \brief  Ein Softwaretimer setzen.
  *
  * \param 	TimerHandle		Nummer des Softwaretimer
  * \param 	TimerMode		Modus des Timers. (single: Einmaldurchauf, peroid: periodischer Durchlauf)
  *	\param	Timeoutms		Zeitdauer in ms
  *	\param	callback		Pointer auf die Callbackfunktion die gerufen wird, wenn der Timer abgelaufen ist
  * \return	0
  *
  */
int TimerSet(TimerName_t TimerHandle, TimerMode_t TimerMode, uint16_t Timeoutms, Callback_t callback);

/**
  * \brief  Ein Softwaretimer starten
  *
  * \param 	TimerHandle		Nummer des Softwaretimer
  * \return	keine
  *
  */
void TimerStart(TimerName_t TimerHandle);

/**
  * \brief  Ein Softwaretimer stoppen
  *
  * \param 	TimerHandle		Nummer des Softwaretimer
  * \return	keine
  *
  */
void TimerStop(TimerName_t TimerHandle);

 /**
  * \brief  Muss durch den Hardwaretimer zyklisch gerufen werden. Berechnet alle Softwaretimer und ruft ggf. die Callbackfunktion
  *
  * \return	0
  *
  */
void TimerInterrupt(void);

#define Timer_H
#endif
