/**
 * de.lohaltec.laufputer: A heart rate meter for Android
 *  
 * <br>Copyright 2010 Albrecht Lohoefener <albrechtloh@gmx.de>
 * Date: 06.10.2010
 *
 * <p>This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation (see COPYING).
 * 
 * <p>This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
package de.lohaltec.laufputer;

import java.util.Queue;
import java.util.TimerTask;
//import android.util.Log;

import android.os.Handler;

public class CheckConnectionTask extends TimerTask
{

//******************************************************************** //
//Class Data.
//******************************************************************** //

// Input data buffer
private Queue<String> inputFiFo;

// Fifo element
private String mFifoElement;

private MainLaufputerActivity mMainLaufputerActivityObject = null;

// Timer counters for connection state
private int noDataCounter = 0;
private int noConnectionCounter = 0;

private Handler mUpdateUIHandler = null;

/**
* Constructor
* @param inputFiFo_ Input buffer
*/
public CheckConnectionTask(Queue<String> inputFiFo_)
{
	inputFiFo = inputFiFo_;
}
		
/**
 * This function is called every x time by the timer
 */
@Override public void run()
{
	if(mMainLaufputerActivityObject != null)
		mUpdateUIHandler = mMainLaufputerActivityObject.getMUpdateUIHandler();
	
	// hier "0" zaehlen: keine daten und "null" zaehlen: keine verbindung
	
	// Fetch a element
	mFifoElement = inputFiFo.peek();
	
	// No data
	if(mFifoElement == "0")
	{
		// remove 0 from FiFo
		inputFiFo.poll();
		
		// reset noConnectionCounter
		noConnectionCounter = 0;
		
		noDataCounter++;
	}
	
	// FiFO empty or no connection
	if(mFifoElement == null)
	{
		noConnectionCounter++;
	}
	
	// Data
	if(mFifoElement != "0" && mFifoElement != null )
	{
		// reset time counters
		noDataCounter = 0;
		noConnectionCounter = 0;
		
		// Post connection state to UI
		sendMessageToUI("Connection ok");
	}
	
	// Until 4s post message to UI
	if(noDataCounter >= 200)
	{
		// Post connection state to UI
		sendMessageToUI("Connection ok, no data received!!");
		
		// reset counter
		noDataCounter = 0;
	}
	
	// Until 2s post message to UI
	if(noConnectionCounter >= 100)
	{
		// Post connection state to UI
		sendMessageToUI("No connection!!");
		
		// reset counter
		noConnectionCounter = 0;
	}
	
}

/**
* Set the MainLaufputerActivity object
* @param mainLaufputerActivityObject MainLaufputerActivity object
*/
public void setMainLaufputerActivityObject(MainLaufputerActivity mainLaufputerActivityObject)
{
	mMainLaufputerActivityObject = mainLaufputerActivityObject;
}

/**
 * Send a message to the UI
 * @param message
 */
private void sendMessageToUI(String message)
{
	if(mMainLaufputerActivityObject != null)
	{
		// Post connection state to UI
		if(mMainLaufputerActivityObject != null)
		{
			mMainLaufputerActivityObject.setMConnectionState(message);
			mUpdateUIHandler.post(mMainLaufputerActivityObject.mUpdateUI);
		}
	}
}

}

