/**
 * de.lohaltec.laufputer: A heart rate meter for Android
 *  
 * <br>Copyright 2010 Albrecht Lohoefener <albrechtloh@gmx.de>
 * Date: 06.09.2010
 *
 * <p>This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation (see COPYING).
 * 
 * <p>This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
package de.lohaltec.laufputer;

import java.util.Queue;
import java.util.TimerTask;

import android.os.Handler;
import android.util.Log;


public class DisplayHeartRateTask extends TimerTask
{

//******************************************************************** //
//Class Data.
//******************************************************************** //

//Input data buffer
private Queue<String> mInputFiFo;

//Fifo element
private String mFifoElement;

private MainLaufputerActivity mMainLaufputerActivityObject = null;

private Handler mUpdateUIHandler = null;

/**
* Constructor
* @param inputFiFo_ Input buffer
*/
public DisplayHeartRateTask(Queue<String> inputFiFo_)
{
	mInputFiFo = inputFiFo_;
}
		
/**
 * This function is called every x time by the timer
 */
@Override public void run()
{
	if(mMainLaufputerActivityObject != null)
		mUpdateUIHandler = mMainLaufputerActivityObject.getMUpdateUIHandler();
	
	// Fetch a element
	mFifoElement = mInputFiFo.peek();
	// Delete the element
	mInputFiFo.poll();
	
	if(mFifoElement != null)
	{
		// Display Heardrate on Screen
		if(!(mFifoElement.equals("0") || mFifoElement.equals("1")))	
		{
			Log.d("Laufputer","Heart rate: " + mFifoElement);
			
			if(mMainLaufputerActivityObject != null)
			{
				// Post heart rate to UI
				mMainLaufputerActivityObject.setMDisplayHeartRate(mFifoElement);
				mUpdateUIHandler.post(mMainLaufputerActivityObject.mUpdateUI);
			}	
		}
		
		// Herztakt anzeigen
		if(mFifoElement.equals("1"))
		{
			mMainLaufputerActivityObject.setHeardOn(true);
			mUpdateUIHandler.post(mMainLaufputerActivityObject.mUpdateUI);
		}
	}
	
}

/**
* Set the MainLaufputerActivity object
* @param mainLaufputerActivityObject MainLaufputerActivity object
*/
public void setMainLaufputerActivityObject(MainLaufputerActivity mainLaufputerActivityObject)
{
	mMainLaufputerActivityObject = mainLaufputerActivityObject;
}

}
