/**
 * de.lohaltec.laufputer: A heart rate meter for Android
 *  
 * <br>Copyright 2010 Albrecht Lohoefener <albrechtloh@gmx.de>
 * Date: 02.10.2010
 *
 * <p>This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation (see COPYING).
 * 
 * <p>This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package de.lohaltec.laufputer;

//import android.util.Log;
import java.util.*;
import java.lang.Math;


public class FskDemod
{
//******************************************************************** //
//Class Data.
//******************************************************************** //

// Offset for the demodulator because the audio samples are in data type short
private final int OFFSET =  32768;

private final double PI = 3.14159265; 
// 0 frequency 
private final int FSK_DEMOD_FREQ0 = 1000;
// 1 frequency
private final int FSK_DEMOD_FREQ1 = 2000;

// Bit duration in ms
private final int BIT_DURATION_MS = 4;
// Sync bit duration in ms
private final float SYNCBIT_DURATION_MS = ((float)3/2 * (float)BIT_DURATION_MS);
// Sync bit variation 2% in ms
private final float SYNCBIT_DURATION_VARIANZ_MS = ((float)SYNCBIT_DURATION_MS * (float)0.02); 
// Break duration in ms
private final int BREAK_DURATION_MS = BIT_DURATION_MS;
//Integrate duration in ms
private final float INT_DURATION_MS = ((float)BIT_DURATION_MS * (float)0.3);

// Threshold value
private final double FSK_DEMOD_THRESHOLD = 0.05 * OFFSET;

//Variables definitions for ncFSKDemod()

private int timeCounter = 0;

//Bit duration in samples
private int BitDuration = 0;
// Sync bit duration in samples
private int SyncBitDuration = 0;
// Sync bit variation 2% in samples
private int SyncBitDarationVarianz = 0; 
// Break duration in samples
private int BreakBitDuration = 0;
//Integrate duration in samples
private int IntegrateDuration = 0;


// Variables definitions for ncFSKDemod()
private short fsk_demod_sin0re = 0;
private short fsk_demod_cos0im = 0;
private short fsk_demod_sin1re = 0;
private short fsk_demod_cos1im = 0;

private long fsk_demod_xsin0re = 0;
private long fsk_demod_xcos0im = 0;
private long fsk_demod_xsin1re = 0;
private long fsk_demod_xcos1im = 0;

private long fsk_demod_int_counter = 0;
private long fsk_demod_isin0re = 0;
private long fsk_demod_icos0im = 0;
private long fsk_demod_isin1re = 0;
private long fsk_demod_icos1im = 0;
private long fsk_demod_isin0re_m = 0;
private long fsk_demod_icos0im_m = 0;
private long fsk_demod_isin1re_m = 0;
private long fsk_demod_icos1im_m = 0;

private long fsk_demod_isin0re2 = 0;
private long fsk_demod_icos0im2 = 0;
private long fsk_demod_isin1re2 = 0;
private long fsk_demod_icos1im2 = 0;

private long s0 = 0;
private long s1 = 0;

private byte y = 0;

//Variables definitions for FskDemodDetectByte()
private byte fsk_demod_byte = 0;
private byte fsk_demod_byte_ready = 0;
private int fsk_demod_sync_duration_counter = 0;
private int fsk_demod_sample_counter = 0;
private int fsk_demod_is_sync = 0;
private int fsk_demod_bit_counter = 0;

//Variables definitions for shiftIntoByte()
private byte fsk_demod_outbyte = 0;
private byte fsk_demod_byte_temp = 0;

//Input data buffer
private Queue<String> inputFiFo;

/**
 * Constructor
 * @param inputFiFo_ Input buffer
 */
public FskDemod(Queue<String> inputFiFo_)
{
	inputFiFo = inputFiFo_;
}

//******************************************************************** //
//Audio Processing.
//******************************************************************** //

/**
* Handle audio input.  This is called by Audio.java
* 
* @param   buffer	 		Audio data that was just read.
* @param   sampleRate       Audio stream sample rate
* @param   inputBlockSize   Buffer block size
*/
public void receiveAudio(short[] buffer, int sampleRate, int inputBlockSize)
{
	float time = 0;
	short sample = 0;
	byte demodSignal = 0;
	
	// Calculate demodulation paramters
	BitDuration = BIT_DURATION_MS * sampleRate / 1000;
	SyncBitDuration = (int)SYNCBIT_DURATION_MS * sampleRate / 1000;
	SyncBitDarationVarianz = (int)SYNCBIT_DURATION_VARIANZ_MS * sampleRate / 1000;
	BreakBitDuration =  BREAK_DURATION_MS * sampleRate / 1000;
	IntegrateDuration = (int)INT_DURATION_MS * sampleRate / 1000;
	
	// Lock to protect updates to these local variables.
	synchronized (buffer) 
	{
		for(int i=0;i<inputBlockSize;i++)
		{
			// Calculate the time stamp
			time = timeCounter / (float) sampleRate;
			
			// convert sample from char to float
			sample = buffer[i];
			
			// Demodulate the audio sample
			demodSignal = ncFSKDemod(sample,time);

			// Decode data
			DecodeByte(demodSignal);
			
			timeCounter++;
			
			if(timeCounter >= sampleRate)
			{
				timeCounter = 0;
				//Log.e("Laufputer","1 sec");
			}
		}
	}
	buffer.notify();
}

/**
* Non coherent FSK demodulator.
* 
* @param   buffer	 		Audio data that was just read.
* @param   sampleRate       Audio stream sample rate
* @param   inputBlockSize   Buffer block size
*/
private byte ncFSKDemod(short sample, float time)
{
	// Threshold usage 
	if (sample > 0 && sample <= FSK_DEMOD_THRESHOLD) sample = 0;
	if (sample < 0 && sample >= -FSK_DEMOD_THRESHOLD) sample = 0;
	
	// Calculation of the compare frequencies
	fsk_demod_sin0re = (short)(OFFSET * Math.sin(2*PI*FSK_DEMOD_FREQ0*time));
	fsk_demod_cos0im = (short)(OFFSET * Math.cos(2*PI*FSK_DEMOD_FREQ0*time));
	fsk_demod_sin1re = (short)(OFFSET * Math.sin(2*PI*FSK_DEMOD_FREQ1*time));
	fsk_demod_cos1im = (short)(OFFSET * Math.cos(2*PI*FSK_DEMOD_FREQ1*time));	

	// Multiply compare frequencies with input sample
	fsk_demod_xsin0re = sample * fsk_demod_sin0re;
	fsk_demod_xcos0im = sample * fsk_demod_cos0im;

	fsk_demod_xsin1re = sample * fsk_demod_sin1re;
	fsk_demod_xcos1im = sample * fsk_demod_cos1im;

	// Integration over some audio samples
	if(fsk_demod_int_counter <= IntegrateDuration)
	{
		// to add up
		fsk_demod_isin0re_m += fsk_demod_xsin0re;
		fsk_demod_icos0im_m += fsk_demod_xcos0im;

		fsk_demod_isin1re_m += fsk_demod_xsin1re;
		fsk_demod_icos1im_m += fsk_demod_xcos1im;
	}
	
	if(fsk_demod_int_counter >= IntegrateDuration)
	{
		// Build averages 
		fsk_demod_isin0re = fsk_demod_isin0re_m / IntegrateDuration;
		fsk_demod_icos0im = fsk_demod_icos0im_m / IntegrateDuration;
		fsk_demod_isin1re = fsk_demod_isin1re_m / IntegrateDuration;
		fsk_demod_icos1im = fsk_demod_icos1im_m / IntegrateDuration;
		
		// Reset sum
		fsk_demod_isin0re_m = 0;
		fsk_demod_icos0im_m = 0;
		
		fsk_demod_isin1re_m = 0;
		fsk_demod_icos1im_m = 0;

		// Reset integrate counter
		fsk_demod_int_counter = 0;
	}
	// Increment integrate counter
	fsk_demod_int_counter++;

	// Squaring
	fsk_demod_isin0re2 = fsk_demod_isin0re * fsk_demod_isin0re;
	fsk_demod_icos0im2 = fsk_demod_icos0im * fsk_demod_icos0im;
	fsk_demod_isin1re2 = fsk_demod_isin1re * fsk_demod_isin1re;
	fsk_demod_icos1im2 = fsk_demod_icos1im * fsk_demod_icos1im;

	// Add imaginary part to real part
	s0 = fsk_demod_isin0re2 + fsk_demod_icos0im2;
	s1 = fsk_demod_isin1re2 + fsk_demod_icos1im2;

	// Decision if value 0 or 1
	if(s0 < s1)
		y = 1;
	else
		y = 0;

	return y;
}

/**
* Decode a byte from the data stream produced by the demodulator
* 
* @param   demod_signal	 	Single sample from the demodulator
*/
private void DecodeByte(byte demod_signal)
{
fsk_demod_byte_ready = 0;
	
	// Sync Erkennung
	if(demod_signal == 1 && fsk_demod_is_sync == 0)
	{
		fsk_demod_sync_duration_counter++;
	}
	if (demod_signal == 0 && fsk_demod_is_sync == 0)
	{
		if(fsk_demod_sync_duration_counter >= SyncBitDuration - SyncBitDarationVarianz
		&& fsk_demod_sync_duration_counter <= SyncBitDuration + SyncBitDarationVarianz)
		{
			//Log.e("Laufputer","Sync");
			fsk_demod_is_sync = 1;
		}
		//if(fsk_demod_sync_duration_counter > 10)
		//	Log.e("Laufputer",String.valueOf(fsk_demod_sync_duration_counter));
		
		fsk_demod_sync_duration_counter = 0;
	}

	// Bit recognition 
	if(fsk_demod_is_sync == 1)
	{
		fsk_demod_sample_counter ++;
		// Read out the first bit
		if(fsk_demod_sample_counter >= (BreakBitDuration + (BitDuration / 2))
		&& fsk_demod_bit_counter == 0)
		{
			fsk_demod_byte = shiftIntoByte(fsk_demod_byte,demod_signal,fsk_demod_bit_counter);
			fsk_demod_bit_counter ++;
			fsk_demod_sample_counter = 0;
		}
		// Read out another Bit
		if(fsk_demod_sample_counter >= BitDuration
		&& fsk_demod_bit_counter > 0)
		{
			fsk_demod_byte = shiftIntoByte(fsk_demod_byte,demod_signal,fsk_demod_bit_counter);
			fsk_demod_bit_counter ++;
			
			// Byte is ready
			if(fsk_demod_bit_counter > 7)
			{
				fsk_demod_byte_ready = fsk_demod_byte;
				fsk_demod_byte = 0;
				fsk_demod_bit_counter = 0;
				fsk_demod_is_sync = 0;
				
				// Save byte in a buffer
				if(fsk_demod_byte_ready>=0)
					inputFiFo.offer(String.valueOf((int)fsk_demod_byte_ready));
				else
					inputFiFo.offer(String.valueOf((int)fsk_demod_byte_ready + 256));
			}
			fsk_demod_sample_counter = 0;
		}
	}
}

/**
* Shift a bit into a byte
* 
* @param   byte_	 		The bit would be shift in this byte
* @param   demodSignal	 	Sample from the demodulator
* @param   bitCounter	 	Bit position in the byte 
*/
private byte shiftIntoByte(byte byte_, byte demodSignal, int bitCounter)
{
	fsk_demod_byte_temp = demodSignal;
	fsk_demod_byte_temp = (byte)(fsk_demod_byte_temp << bitCounter);
	fsk_demod_outbyte = (byte)(byte_ | fsk_demod_byte_temp);
	
	return fsk_demod_outbyte;
}

}
