/**
 * de.lohaltec.laufputer: A heart rate meter for Android
 *  
 * <br>Copyright 2010 Albrecht Lohoefener <albrechtloh@gmx.de>
 * Date: 07.09.2010
 *
 * <p>This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation (see COPYING).
 * 
 * <p>This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package de.lohaltec.laufputer;

import java.util.LinkedList;
import java.util.Queue;

import  android.app.Application;

public class Laufputer extends Application
{
// ******************************************************************** //
// Class Data.
// ******************************************************************** //
	
// Create object pointer
private FskDemod fskDemod = null;
private Audio audio = null;

private CheckConnection checkConnection = null;
private DisplayHeartRate displayHeartRate = null;

//Input data buffer
private Queue<String> inputFiFo = null;

/**
 * Program init
 * Called only on the program start 
 */
@Override
public void onCreate()
{
   super.onCreate();
	    
   // Program init
   
   // Create input buffer
   inputFiFo = new LinkedList<String>();
   
   // Create audio objects
   fskDemod = new FskDemod(inputFiFo);
   audio = new Audio(fskDemod);
   
   // set audio properties 
   //audio.setSampleRate(22050);
   audio.setSampleRate(11025);
   audio.setBlockSize(512);
   
   // Setup screen
   displayHeartRate = new DisplayHeartRate(inputFiFo);
   
   // Setup Check connection
   checkConnection = new CheckConnection(inputFiFo);
   
}

/**
 * Get the audio object
 * @return audio object
 */
public Audio getAudioObject()
{
	return audio;
}

/**
 * Get the displayHeartRate object
 * @return displayHeartRate object
 */
public DisplayHeartRate getDisplayHeartRateObject()
{
	return displayHeartRate;
}

/**
 * Get the checkConnection object
 * @return checkConnection object
 */
public CheckConnection getCheckConnectionObject()
{
	return checkConnection;
}

/**
 * Get the input buffer implement as a fifo
 * @return input buffer
 */
public Queue<String> getInputFiFo()
{
	return inputFiFo;
}

}