/**
 * de.lohaltec.laufputer: A heart rate meter for Android
 *  
 * <br>Copyright 2010 Albrecht Lohoefener <albrechtloh@gmx.de>
 * Date: 02.09.2010
 *
 * <p>This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation (see COPYING).
 * 
 * <p>This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package de.lohaltec.laufputer;

import de.lohaltec.laufputer.R;
import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.SystemClock;
import android.widget.TextView;
import android.widget.ImageView;
import android.widget.Button;
import android.widget.Chronometer;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;

public class MainLaufputerActivity extends Activity
{
// ******************************************************************** //
// Class Data.
// ******************************************************************** //

// Create object pointer
private Audio audio = null;

private Chronometer totalRunTimeChronometer = null;
private Chronometer splitTimeChronometer = null;

private TextView HeartRateView = null;
private TextView ConnectionStateView = null;
private ImageView HeartView = null;

// Need handler for callbacks to the UI thread
private final Handler mUpdateUIHandler = new Handler();

// Create runnable for updating UI
public final Runnable mUpdateUI = new Runnable() {
    public void run() {
    	updateUI();
    }
};

private String mDisplayHeartRate;
private String mConnectionState;
private boolean mIsHardOn = false;
private int mIsHardOnCounter = 0;

private boolean mIsRunning = false;

/**
 * Called when the activity is first created. 
 */
@Override
public void onCreate(Bundle savedInstanceState)
{
	super.onCreate(savedInstanceState);
    setContentView(R.layout.main);
               
    Button startButton = (Button) findViewById(R.id.StartButton);
    startButton.setOnClickListener(new OnClickListener()
    {
    	//@Override
        public void onClick(View v)
        {
        	startButtonClick();
        }
    });
    
    Button stopButton = (Button) findViewById(R.id.StopButton);
    stopButton.setOnClickListener(new OnClickListener()
    {
    	//@Override
        public void onClick(View v)
        {
        	stopButtonClick();
        }
    });
    
    // Get global objects
    audio = ((Laufputer)getApplication()).getAudioObject();
    
    // Get DisplayHeartRate object
    DisplayHeartRate displayHeartRate = ((Laufputer)getApplication()).getDisplayHeartRateObject();
    // Set this object to DisplayHeartRate, because DisplayHeartRate need the handler mUpdateUIHandler
    displayHeartRate.setMainLaufputerActivityObject(this);
    
    // Get DisplayHeartRate object
    CheckConnection checkConnection = ((Laufputer)getApplication()).getCheckConnectionObject();
    // Set this object to DisplayHeartRate, because DisplayHeartRate need the handler mUpdateUIHandler
    checkConnection.setMainLaufputerActivityObject(this);
    
    // Load default value in the HeartRateView
	HeartRateView = (TextView) findViewById(R.id.HeartRateView);
	HeartRateView.setText("0");
	mDisplayHeartRate = "0";
	
	ConnectionStateView = (TextView) findViewById(R.id.ConnectionStateView);
	
	HeartView = (ImageView) findViewById(R.id.HeartView);
	
	// Setup chronomter
	totalRunTimeChronometer = (Chronometer) findViewById(R.id.TotalRunTimeChronometer);
	splitTimeChronometer = (Chronometer) findViewById(R.id.SplitTimeChronometer);
	
}

/**
 *  Save activity state in a bundle
 *  Called when from the onStop() function
 */
@Override
protected void onSaveInstanceState(Bundle outState) {
	super.onSaveInstanceState(outState);
	
	// Stop chronometer
	totalRunTimeChronometer.stop();
	splitTimeChronometer.stop();
	
	// Save base into the bundle
	outState.putLong("totalRunTimeChronometer", totalRunTimeChronometer.getBase());
	outState.putLong("splitTimeChronometer", splitTimeChronometer.getBase());
	
	// Save HeartRateView
	outState.putCharSequence("HeartRateView",HeartRateView.getText());
	// Save ConnectionStateView
	outState.putCharSequence("ConnectionStateView",ConnectionStateView.getText());
	
	// Save mConnectionState
	outState.putString("mConnectionState", mConnectionState);
	// Save mDisplayHeartRate
	outState.putString("mDisplayHeartRate", mDisplayHeartRate);
	
	// Save running state
	outState.putBoolean("mIsRunning", mIsRunning);
}

/**
 *  Restore a activity state from a bundle
 *  Called when from the onStart() function
 */
@Override
protected void onRestoreInstanceState(Bundle savedInstanceState) {
	super.onRestoreInstanceState(savedInstanceState);
	
	// Get base from the bundle
	long totalRunTimeChronometerBase = savedInstanceState.getLong("totalRunTimeChronometer");
	long splitTimeChronometerBase = savedInstanceState.getLong("splitTimeChronometer");
	// Restore chronometer base
	totalRunTimeChronometer.setBase(totalRunTimeChronometerBase);
	splitTimeChronometer.setBase(splitTimeChronometerBase);
	// Start chronometer
	totalRunTimeChronometer.start();
	splitTimeChronometer.start();
	
	// Restore HeartRateView
	HeartRateView.setText(savedInstanceState.getCharSequence("HeartRateView"));
	// Restore ConnectionStateView
	ConnectionStateView.setText(savedInstanceState.getCharSequence("ConnectionStateView"));
	
	// Restore mConnectionState
	mConnectionState = savedInstanceState.getString("mConnectionState");
	// Restore mDisplayHeartRate
	mDisplayHeartRate = savedInstanceState.getString("mDisplayHeartRate");
	
	// Restore running state
	mIsRunning = savedInstanceState.getBoolean("mIsRunning");
	
}

/**
 *  Start the audio processing and the chronometers
 *  Called from the startButton OnClickListener 
 */
public void startButtonClick()
{	
	if(!mIsRunning)
	{
		Log.d("Laufputer","start audio recording");
	    // start recording
		if(audio != null)
			audio.audioStart();
	    
	    // Error handling
	    int error = audio.getError();
	    if (error != 0)
	    	Log.e("Laufputer","Audio recording error: " + String.valueOf(error));
	    
	    // Start run time chronometer
	    // Reset chronometer
	    totalRunTimeChronometer.setBase(SystemClock.elapsedRealtime());
	    // Start chronometer
	    totalRunTimeChronometer.start();
	   
	    mIsRunning = true;
	}
	
	// Start split timer chronometer
    // Reset chronometer
    splitTimeChronometer.setBase(SystemClock.elapsedRealtime());
    // Start chronometer
    splitTimeChronometer.start();
    
}

/**
 *  Stop the audio processing and the chronometers
 *  Called from the stopButton OnClickListener 
 */
public void stopButtonClick()
{
	if(mIsRunning)
	{
		Log.d("Laufputer","stop audio recording");
	  	// stop recording
		if(audio != null)
			audio.audioStop();
		
		// clear heart rate view 
		HeartRateView.setText("0");
		mDisplayHeartRate = "0";
		
		// Stop run time chronometer
	    totalRunTimeChronometer.stop();
	    
	    // Stop split timer chronometer
	    splitTimeChronometer.stop();
	    
	    mIsRunning = false;
	    
	    
	}
}

/**
 *  Get the UpdateUIHandler object
 *  @return UpdateUIHandler object
 */
public Handler getMUpdateUIHandler()
{
	return mUpdateUIHandler;
}

/**
 *  Set variable mDisplayHeartRate
 *  @param DisplayHeartRate Heart rate
 */
public void setMDisplayHeartRate(String DisplayHeartRate)
{
	mDisplayHeartRate = DisplayHeartRate;
}

/**
 *  Set variable mConnectionState
 *  @param ConnectionState State of the connection
 */
public void setMConnectionState(String ConnectionState)
{
	mConnectionState = ConnectionState;
}

/**
 *  Set variable mConnectionState
 *  @param ConnectionState State of the connection
 */
public void setHeardOn(boolean isHardOn)
{
	mIsHardOn = isHardOn;
}

/**
 *  Update the UI
*/
protected void updateUI()
{
	
	// Update HeartRateView
	HeartRateView.setText(mDisplayHeartRate);
	
	// Update ConnectionStateView
	ConnectionStateView.setText(mConnectionState);
	
	/*if(mIsHardOn)
	{
		// Herz anzeigen
	    HeartView.setVisibility(1);
		
		mIsHardOnCounter++;
		
		if(mIsHardOnCounter==6)
		{
			mIsHardOnCounter = 0;
			
			mIsHardOn = false;
			
			// Herz nicht anzeigen
		    HeartView.setVisibility(4);
		}
	}*/
	
}

}
